/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.artifacts.bundu.bezugswst;

import java.util.HashSet;
import java.util.Set;

import org.dive4elements.artifactdatabase.state.ArtifactAndFacet;
import org.dive4elements.artifactdatabase.state.Facet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.bundu.BunduResultType;
import org.dive4elements.river.artifacts.common.AbstractProcessor;
import org.dive4elements.river.artifacts.common.AbstractResultType;
import org.dive4elements.river.artifacts.common.GeneralResultType;
import org.dive4elements.river.artifacts.resources.Resources;
import org.dive4elements.river.artifacts.sinfo.common.SInfoResultType;
import org.dive4elements.river.exports.DiagramGenerator;
import org.dive4elements.river.exports.LongitudinalSectionGenerator;
import org.dive4elements.river.themes.ThemeDocument;

/**
 * Processor to generate the facets and data series of a bundu bezugswst depth longitudinal section
 *
 * @author Matthias Schäfer
 *
 */
public final class BezugswstHeightProcessor extends AbstractProcessor {

    private static final String FACET_W = "bundu_facet_w";

    private static final String FACET_W_DESCRIPTION = "bundu_facet_w.description";

    private static final String FACET_CHANNELMIN = "bundu_facet_channelmin";

    private static final String FACET_CHANNELMIN_DESCRIPTION = "bundu_facet_channelmin.description";

    private static final String FACET_BEDHEIGHT = "bundu_facet_bedheight";

    private static final String FACET_BEDHEIGHT_DESCRIPTION = "bundu_facet_bedheight.description";

    public static final String FACET_Q = "bundu_facet_q";

    private static final String FACET_Q_DESCRIPTION = "bundu_facet_q.description";

    private static final String AXIS_LABEL = LongitudinalSectionGenerator.I18N_YAXIS_LABEL;

    private static final String Q_AXIS_LABEL = LongitudinalSectionGenerator.I18N_2YAXIS_LABEL;

    private static final Set<String> HANDLED_FACET_TYPES = new HashSet<>();

    static {
        HANDLED_FACET_TYPES.add(FACET_W);
        HANDLED_FACET_TYPES.add(FACET_CHANNELMIN);
        HANDLED_FACET_TYPES.add(FACET_BEDHEIGHT);
        HANDLED_FACET_TYPES.add(FACET_Q);
    }

    public BezugswstHeightProcessor() {
        super(AXIS_LABEL, HANDLED_FACET_TYPES);
    }

    public static Facet createWFacet(final CallContext context, final String hash, final String id, final int facetIndex,
            final int resultIndex, final String seriesName) {

        final String description = Resources.getMsg(context.getMeta(), FACET_W_DESCRIPTION, FACET_W_DESCRIPTION, seriesName);
        return new BezugswstResultFacet(facetIndex, resultIndex, FACET_W, description, AXIS_LABEL, id, hash);
    }

    public static Facet createChannelminFacet(final CallContext context, final String hash, final String id, final int facetIndex,
            final int resultIndex) {

        final String description = Resources.getMsg(context.getMeta(), FACET_CHANNELMIN_DESCRIPTION, FACET_CHANNELMIN_DESCRIPTION);
        return new BezugswstResultFacet(facetIndex, resultIndex, FACET_CHANNELMIN, description, AXIS_LABEL, id, hash);
    }

    public static Facet createBedheightFacet(final CallContext context, final String hash, final String id, final int facetIndex,
            final int resultIndex, final String seriesName) {

        final String description = Resources.getMsg(context.getMeta(), FACET_BEDHEIGHT_DESCRIPTION, FACET_BEDHEIGHT_DESCRIPTION, seriesName);
        return new BezugswstResultFacet(facetIndex, resultIndex, FACET_BEDHEIGHT, description, AXIS_LABEL, id, hash);
    }

    @Override
    protected String generateSeries(final DiagramGenerator generator, final ArtifactAndFacet bundle, final ThemeDocument theme, final boolean visible) {

        return buildSeriesForType(generator, bundle, theme, visible, doGetType(bundle.getFacetName()), GAP_DISTANCE);
    }

    protected AbstractResultType doGetType(final String facetName) {

        if (FACET_W.contentEquals(facetName))
            return BunduResultType.bezugswst;
        if (FACET_CHANNELMIN.contentEquals(facetName))
            return BunduResultType.channelLowerEdge;
        if (FACET_BEDHEIGHT.contentEquals(facetName))
            return SInfoResultType.meanBedHeight;
        if (FACET_Q.contentEquals(facetName))
            return GeneralResultType.dischargeQwithUnit;

        final String error = String.format("Unknown facet name: %s", facetName);
        throw new UnsupportedOperationException(error);
    }
}